@@ -1,3 +1,4 @@
+use self::util::BoolOrObject;
 use crate::{builder::PassBuilder, SwcComments, SwcImportResolver};
 use anyhow::{bail, Context, Error};
 use dashmap::DashMap;
@@ -39,8 +40,6 @@ use swc_ecma_transforms::{
 };
 use swc_ecma_visit::Fold;
 
-use self::util::BoolOrObject;
-
 #[cfg(test)]
 mod tests;
 pub mod util;
@@ -213,6 +212,8 @@ impl Options {
         let syntax = syntax.unwrap_or_default();
         let mut transform = transform.unwrap_or_default();
 
+        let preserve_comments = js_minify.as_ref().map(|v| v.format.comments.clone());
+
         if syntax.typescript() {
             transform.legacy_decorator = true;
         }
@@ -310,6 +311,7 @@ impl Options {
             input_source_map: self.config.input_source_map.clone(),
             output_path: output_path.map(|v| v.to_path_buf()),
             source_file_name,
+            preserve_comments,
         }
     }
 }
@@ -509,6 +511,9 @@ pub struct JsMinifyOptions {
     #[serde(default)]
     pub mangle: BoolOrObject<MangleOptions>,
 
+    #[serde(default)]
+    pub format: JsMinifyFormatOptions,
+
     #[serde(default)]
     pub ecma: TerserEcmaVersion,
 
@@ -534,6 +539,27 @@ pub struct JsMinifyOptions {
     pub output_path: Option<String>,
 }
 
+/// `jsc.minify.format`.
+#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[serde(deny_unknown_fields, rename_all = "camelCase")]
+pub struct JsMinifyFormatOptions {
+    pub comments: BoolOrObject<JsMinifyCommentOption>,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub enum JsMinifyCommentOption {
+    #[serde(rename = "some")]
+    PreserveSomeComments,
+    #[serde(rename = "all")]
+    PreserveAllComments,
+}
+
+impl Default for JsMinifyCommentOption {
+    fn default() -> Self {
+        JsMinifyCommentOption::PreserveSomeComments
+    }
+}
+
 impl Config {
     /// Adjust config for `file`.
     ///
@@ -639,6 +665,8 @@ pub struct BuiltConfig<P: swc_ecma_visit::Fold> {
     pub output_path: Option<PathBuf>,
 
     pub source_file_name: Option<String>,
+
+    pub preserve_comments: Option<BoolOrObject<JsMinifyCommentOption>>,
 }
 
 #[derive(Debug, Default, Clone, Serialize, Deserialize)]
@@ -941,6 +969,7 @@ impl Merge for JsMinifyOptions {
     fn merge(&mut self, from: &Self) {
         self.compress.merge(&from.compress);
         self.mangle.merge(&from.mangle);
+        self.format.merge(&from.format);
         self.ecma.merge(&from.ecma);
         self.keep_classnames |= from.keep_classnames;
         self.keep_fnames |= from.keep_fnames;
@@ -968,6 +997,16 @@ impl Merge for MangleOptions {
     }
 }
 
+impl Merge for JsMinifyFormatOptions {
+    fn merge(&mut self, from: &Self) {
+        self.comments.merge(&from.comments);
+    }
+}
+
+impl Merge for JsMinifyCommentOption {
+    fn merge(&mut self, _: &Self) {}
+}
+
 impl Merge for ManglePropertiesOptions {
     fn merge(&mut self, from: &Self) {
         self.undeclared |= from.undeclared;
